@@ -586,6 +586,15 @@ Stream cv::cuda::StreamAccessor::wrapStream(cudaStream_t stream)
 
 #endif
 
+Stream cv::cuda::wrapStream(size_t cudaStreamMemoryAddress) {
+#ifndef HAVE_CUDA
+    CV_UNUSED(cudaStreamMemoryAddress);
+    throw_no_cuda();
+#else
+    return cv::cuda::StreamAccessor::wrapStream(reinterpret_cast<cudaStream_t>(cudaStreamMemoryAddress));
+#endif
+}
+
 /////////////////////////////////////////////////////////////
 /// StackAllocator
 
