@@ -39,8 +39,14 @@
 //
 //M*/
 
-#include <opencv2/core.hpp>
+#if !defined(BUILD_PLUGIN)
 #include "cvconfig.h"
+#else
+#include <opencv2/core/cvdef.h>
+#undef __OPENCV_BUILD  // allow public API only
+#endif
+
+#include <opencv2/core.hpp>
 
 #ifndef CV_OCL4DNN
 #define CV_OCL4DNN 0
@@ -83,4 +89,4 @@
 #include <opencv2/dnn/all_layers.hpp>
 #include <opencv2/dnn/shape_utils.hpp>
 
-#include "dnn_common.hpp"
+#include "dnn_common.hpp"
\ No newline at end of file
