@@ -35,7 +35,7 @@ namespace cvflann
      * \deprecated Provided for backward compatibility
     */
     flann_distance_t flann_distance_type_ = FLANN_DIST_L2;
-    flann_distance_t flann_distance_type() { return flann_distance_type_; }
+    CV_DEPRECATED flann_distance_t flann_distance_type() { return flann_distance_type_; }
 
     /**
      * Set distance type to used
