@@ -1,17 +1,14 @@
+// This file is part of OpenCV project.
+// It is subject to the license terms in the LICENSE file found in the top-level directory
+// of this distribution and at http://opencv.org/license.html.
 #include "perf_precomp.hpp"
 
-using namespace std;
-using namespace cv;
-using namespace perf;
-using namespace testing;
-using std::tr1::make_tuple;
-using std::tr1::get;
-
+namespace opencv_test {
 
 CV_ENUM(BorderMode, BORDER_CONSTANT, BORDER_REPLICATE, BORDER_REFLECT_101)
 
-typedef TestBaseWithParam< tr1::tuple<Size, int, BorderMode> > TestFilter2d;
-typedef TestBaseWithParam< tr1::tuple<String, int> > Image_KernelSize;
+typedef TestBaseWithParam< tuple<Size, int, BorderMode> > TestFilter2d;
+typedef TestBaseWithParam< tuple<string, int> > Image_KernelSize;
 
 PERF_TEST_P( TestFilter2d, Filter2d,
              Combine(
@@ -37,7 +34,34 @@ PERF_TEST_P( TestFilter2d, Filter2d,
 
     declare.in(src, WARMUP_RNG).out(dst).time(20);
 
-    TEST_CYCLE() filter2D(src, dst, CV_8UC4, kernel, Point(1, 1), 0., borderMode);
+    TEST_CYCLE() cv::filter2D(src, dst, CV_8UC4, kernel, Point(1, 1), 0., borderMode);
+
+    SANITY_CHECK(dst, 1);
+}
+
+PERF_TEST_P(TestFilter2d, DISABLED_Filter2d_ovx,
+            Combine(
+                Values(Size(320, 240), sz1080p),
+                Values(3, 5),
+                Values(BORDER_CONSTANT, BORDER_REPLICATE)
+            )
+)
+{
+    Size sz;
+    int borderMode, kSize;
+    sz = get<0>(GetParam());
+    kSize = get<1>(GetParam());
+    borderMode = get<2>(GetParam());
+
+    Mat src(sz, CV_8UC1);
+    Mat dst(sz, CV_16SC1);
+
+    Mat kernel(kSize, kSize, CV_16SC1);
+    randu(kernel, -3, 10);
+
+    declare.in(src, WARMUP_RNG).out(dst).time(20);
+
+    TEST_CYCLE() cv::filter2D(src, dst, CV_16SC1, kernel, Point(kSize / 2, kSize / 2), 0., borderMode);
 
     SANITY_CHECK(dst, 1);
 }
@@ -48,7 +72,7 @@ PERF_TEST_P( Image_KernelSize, GaborFilter2d,
                  Values(16, 32, 64) )
              )
 {
-    String fileName = getDataPath(get<0>(GetParam()));
+    string fileName = getDataPath(get<0>(GetParam()));
     Mat sourceImage = imread(fileName, IMREAD_GRAYSCALE);
     if( sourceImage.empty() )
     {
@@ -67,8 +91,10 @@ PERF_TEST_P( Image_KernelSize, GaborFilter2d,
 
     TEST_CYCLE()
     {
-        filter2D(sourceImage, filteredImage, CV_32F, gaborKernel);
+        cv::filter2D(sourceImage, filteredImage, CV_32F, gaborKernel);
     }
 
-    SANITY_CHECK(filteredImage, 1e-3);
+    SANITY_CHECK(filteredImage, 1e-6, ERROR_RELATIVE);
 }
+
+} // namespace
