@@ -61,6 +61,13 @@ int warpAffineBlockline(int *adelta, int *bdelta, short* xy, short* alpha, int X
 #endif
 }
 
+namespace opt_LASX
+{
+#if CV_TRY_LASX
+int warpAffineBlockline(int *adelta, int *bdelta, short* xy, short* alpha, int X0, int Y0, int bw);
+#endif
+}
+
 namespace opt_SSE4_1
 {
 #if CV_TRY_SSE4_1
@@ -75,7 +82,7 @@ public:
     static Ptr<WarpPerspectiveLine_SSE4> getImpl(const double *M);
     virtual void processNN(const double *M, short* xy, double X0, double Y0, double W0, int bw) = 0;
     virtual void process(const double *M, short* xy, short* alpha, double X0, double Y0, double W0, int bw) = 0;
-    virtual ~WarpPerspectiveLine_SSE4() {};
+    virtual ~WarpPerspectiveLine_SSE4() {}
 };
 #endif
 }
