@@ -319,19 +319,31 @@ class JSWrapperGenerator(object):
             sys.exit(-1)
         self.classes[class_info.name] = class_info
 
-        if class_info.bases:
-            chunks = class_info.bases[0].split('::')
-            base = '_'.join(chunks)
-            while base not in self.classes and len(chunks) > 1:
-                del chunks[-2]
+    def resolve_class_inheritance(self):
+        new_classes = {}
+        for name, class_info in self.classes.items():
+
+            if not hasattr(class_info, 'bases'):
+                new_classes[name] = class_info
+                continue # not class
+
+            if class_info.bases:
+                chunks = class_info.bases[0].split('::')
                 base = '_'.join(chunks)
-            if base not in self.classes:
-                print("Generator error: unable to resolve base %s for %s"
-                      % (class_info.bases[0], class_info.name))
-                sys.exit(-1)
-            else:
-                class_info.bases[0] = "::".join(chunks)
-                class_info.isalgorithm |= self.classes[base].isalgorithm
+                while base not in self.classes and len(chunks) > 1:
+                    del chunks[-2]
+                    base = '_'.join(chunks)
+                if base not in self.classes:
+                    print("Generator error: unable to resolve base %s for %s"
+                        % (class_info.bases[0], class_info.name))
+                    sys.exit(-1)
+                else:
+                    class_info.bases[0] = "::".join(chunks)
+                    class_info.isalgorithm |= self.classes[base].isalgorithm
+
+            new_classes[name] = class_info
+
+        self.classes = new_classes
 
     def split_decl_name(self, name):
         chunks = name.split('.')
@@ -470,7 +482,7 @@ class JSWrapperGenerator(object):
                     ret_type = type_dict[ptr_type]
             for key in type_dict:
                 if key in ret_type:
-                    ret_type = re.sub('(^|[^\w])' + key + '($|[^\w])', type_dict[key], ret_type)
+                    ret_type = re.sub(r"\b" + key + r"\b", type_dict[key], ret_type)
             arg_types = []
             unwrapped_arg_types = []
             for arg in variant.args:
@@ -658,7 +670,7 @@ class JSWrapperGenerator(object):
                     # Replace types. Instead of ret_type.replace we use regular
                     # expression to exclude false matches.
                     # See https://github.com/opencv/opencv/issues/15514
-                    ret_type = re.sub('(^|[^\w])' + key + '($|[^\w])', type_dict[key], ret_type)
+                    ret_type = re.sub(r"\b" + key + r"\b", type_dict[key], ret_type)
             if variant.constret and ret_type.startswith('const') == False:
                 ret_type = 'const ' + ret_type
             if variant.refret and ret_type.endswith('&') == False:
@@ -759,6 +771,8 @@ class JSWrapperGenerator(object):
                 else:  # class/global function
                     self.add_func(decl)
 
+        self.resolve_class_inheritance()
+
         # step 2: generate bindings
         # Global functions
         for ns_name, ns in sorted(self.namespaces.items()):
@@ -812,6 +826,7 @@ class JSWrapperGenerator(object):
         for name, class_info in sorted(self.classes.items()):
             class_bindings = []
             if not name in white_list:
+                #print('Not in whitelist: "{}" from ns={}'.format(name, ns_name))
                 continue
 
             # Generate bindings for methods
