@@ -114,4 +114,30 @@ TEST(ParallelFeaturesFinder, IsSameWithSerial)
     }
 }
 
+TEST(RangeMatcher, MatchesRangeOnly)
+{
+    Ptr<Feature2D> finder = ORB::create();
+
+    Mat img0 = imread(string(cvtest::TS::ptr()->get_data_path()) + "stitching/a1.png", IMREAD_GRAYSCALE);
+    Mat img1 = imread(string(cvtest::TS::ptr()->get_data_path()) + "stitching/a2.png", IMREAD_GRAYSCALE);
+    Mat img2 = imread(string(cvtest::TS::ptr()->get_data_path()) + "stitching/a3.png", IMREAD_GRAYSCALE);
+
+    vector<detail::ImageFeatures> features(3);
+
+    computeImageFeatures(finder, img0, features[0]);
+    computeImageFeatures(finder, img1, features[1]);
+    computeImageFeatures(finder, img2, features[2]);
+
+    vector<detail::MatchesInfo> pairwise_matches;
+    Ptr<detail::FeaturesMatcher> matcher = makePtr<detail::BestOf2NearestRangeMatcher>(1);
+
+    (*matcher)(features, pairwise_matches);
+
+    // matches[1] will be image 0 and image 1, should have non-zero confidence
+    EXPECT_NE(pairwise_matches[1].confidence, .0);
+
+    // matches[2] will be image 0 and image 2, should have zero confidence due to range_width=1
+    EXPECT_DOUBLE_EQ(pairwise_matches[2].confidence, .0);
+}
+
 }} // namespace
