@@ -4,13 +4,13 @@
 
 (ns simple-sample.core
   (:import [org.opencv.core Point Rect Mat CvType Size Scalar]
-           org.opencv.highgui.Highgui
+           org.opencv.imgcodecs.Imgcodecs
            org.opencv.imgproc.Imgproc))
 
 (defn -main [& args]
-  (let [lena (Highgui/imread "resources/images/lena.png")
+  (let [lena (Imgcodecs/imread "resources/images/lena.png")
         blurred (Mat. 512 512 CvType/CV_8UC3)]
     (print "Blurring...")
     (Imgproc/GaussianBlur lena blurred (Size. 5 5) 3 3)
-    (Highgui/imwrite "resources/images/blurred.png" blurred)
+    (Imgcodecs/imwrite "resources/images/blurred.png" blurred)
     (println "done!")))
